#include "Encrypted_File.h"
#include "FileMan.h"
#include "Debug.h"
#include "Language_Defines.h"


BOOLEAN LoadEncryptedData(HWFILE File, wchar_t* DestString, UINT32 Seek, UINT32 ReadBytes)
{
	if (!FileSeek(File, Seek, FILE_SEEK_FROM_START))
	{
		DebugMsg(TOPIC_JA2, DBG_LEVEL_3, "LoadEncryptedData: Failed FileSeek");
		return FALSE;
	}

	UINT16 Str[ReadBytes / 2];
	if (!FileRead(File, Str, sizeof(Str)))
	{
		DebugMsg(TOPIC_JA2, DBG_LEVEL_3, "LoadEncryptedData: Failed FileRead");
		return FALSE;
	}

	Str[ReadBytes / 2 - 1] = '\0';
	for (const UINT16* i = Str; *i != '\0'; ++i)
	{
		/* "Decrypt" the ROT-1 "encrypted" data */
		wchar_t c = (*i > 33 ? *i - 1 : *i);

#if defined RUSSIAN
		/* The Russian data files are incorrectly encoded. The original texts seem to
		 * be encoded in CP1251, but then they were converted from CP1252 (!) to
		 * UTF-16 to store them in the data files. Undo this damage here. */
		if (0xC0 <= c && c <= 0xFF) c += 0x0350;
#else
#	if defined POLISH
		/* The Polish data files are incorrectly encoded. The original texts seem to
		 * be encoded in CP1250, but then they were converted from CP1252 (!) to
		 * UTF-16 to store them in the data files. Undo this damage here.
		 * Also the format code for centering texts differs. */
		switch (c)
		{
			case 143: c = 0x0179; break;
			case 163: c = 0x0141; break;
			case 165: c = 0x0104; break;
			case 175: c = 0x017B; break;
			case 179: c = 0x0142; break;
			case 182: c = 179;    break; // not a char, but a format code (centering)
			case 185: c = 0x0105; break;
			case 191: c = 0x017C; break;
			case 198: c = 0x0106; break;
			case 202: c = 0x0118; break;
			case 209: c = 0x0143; break;
			case 230: c = 0x0107; break;
			case 234: c = 0x0119; break;
			case 241: c = 0x0144; break;
			case 338: c = 0x015A; break;
			case 339: c = 0x015B; break;
			case 376: c = 0x017A; break;
		}
#	endif

		/* Cyrillic texts (by Ivan Dolvich) in the non-Russian versions are encoded
		 * in some wild manner. Undo this damage here. */
		if (0x044D <= c && c <= 0x0452) // cyrillic A to IE
		{
			c += -0x044D + 0x0410;
		}
		else if (c == 0x0453) // cyrillic IO
		{
			c = 0x0401;
		}
		else if (0x0454 <= c && c <= 0x0467) // cyrillic ZHE to SHCHA
		{
			c += -0x0454 + 0x0416;
		}
		else if (0x0468 <= c && c <= 0x046C) // cyrillic YERU to YA
		{
			c += -0x0468 + 0x042B;
		}
#endif

		*DestString++ = c;
	}
	*DestString = L'\0';

	return TRUE;
}


BOOLEAN LoadEncryptedDataFromFile(const char* Filename, wchar_t DestString[], UINT32 Seek, UINT32 ReadBytes)
{
	HWFILE File = FileOpen(Filename, FILE_ACCESS_READ);
	if (File == 0)
	{
		DebugMsg(TOPIC_JA2, DBG_LEVEL_3, "LoadEncryptedDataFromFile: Failed to FileOpen");
		return FALSE;
	}

	BOOLEAN Ret = LoadEncryptedData(File, DestString, Seek, ReadBytes);
	FileClose(File);
	return Ret;
}
